// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       dataset
//  TYPE:       c++ code
//  PROJECT:    lodestar
//  CONTENT:    class DataSet, class Data
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================

#include "dataset.h"

//----------------------------------------------------------------------------
//   class Data
//----------------------------------------------------------------------------

Data::Data()
{
    status=0;
    index=-1;
}

//---Data::print--------------------------------------------------------------
// Input : FILE *fd     : VRML output file descriptor
//         int fid      :
// fid...field-identifier  tells the method which variables have to be printed

void Data::print(FILE *fd, int fid)    //virtual
{
}

//----------------------------------------------------------------------------
//   class DataSet 
//----------------------------------------------------------------------------

//---DataSet::DataSet---------------------------------------------------------
// Input :
// Output:

DataSet::DataSet()
{
    count=0;
    countUncompressed=0;
    refArray = new ReferenceArray();
}

//---DataSet::~DataSet--------------------------------------------------------
// Input :
// Output:

DataSet::~DataSet()
{
    delete refArray;
}

//---DataSet::getIndex--------------------------------------------------------
// Input : int index       : index for array
// Output: Data *          : 
// get element by index (insertion order)

Data *DataSet::getIndex(int index)
{
    return (Data *)((*refArray)[index].ref);
}

//---DataSet::getMaxIndex-----------------------------------------------------
// Input :
// Output:
//returns count of inserted elements-1 (with doublets !)

int DataSet::getMaxIndex()
{
    return refArray->getSize()-1;
}

//---DataSet::insertArray-----------------------------------------------------
// Input :
// Output:
// add a Reference to d to the dynamic array refArray
// this allows picking an element by index in insertion order

void DataSet::insertArray(Data *d)
{
    Reference h = Reference(d);
    refArray->append(h);
}

//---DataSet::printSet--------------------------------------------------------
// Input : FILE *fd     : VRML output file descriptor
//         int indent   : indent for output file
//         int fid      : 
// Output:
// prints out the contents of the inserted elements by calling their print-
// method. The elements are printed generally in insertion order BUT double
// defined elements are printed only once. exception: if countUncompressed>0
// then doublets are printed until the count of the elements printed is
// equal or greater than countUncompressed. 
// e.g. the first 10 elements (X) are all the same (this means the first 10
// references in refArray points to the same element) and countUncompressed=6
// printSet prints X six times and than all other elements without doublets
// uses global variables global_cpl and global_floatcpl for formating the
// output with carriage returns

void DataSet::printSet(FILE *fd, int indent, int fid)
{
    int i;
    int co = refArray->getSize();
    Data *data;
    int first=1;
    global_cpl=0;

    if (countUncompressed>getMaxIndex()+1) countUncompressed=getMaxIndex()+1;

    for (i=0; i<co; i++) 
    {
        data=getIndex(i);
        data->status &= ~DATASET_STAT_DONE;
    }

    for (i=0; i<co; i++)
    {
        data=getIndex(i); 

        if ((data->status & DATASET_STAT_DONE)==0 || (i<countUncompressed))
        {
            if (first) first=0;
            else
            {
                if (global_cpl>=global_floatcpl)
                {
                    fprintf(fd,",\n");
                    printIndent(fd,indent);
                    global_cpl=0;
                }
                else fprintf(fd,",");
            }

            data->print(fd,fid);
            data->status |= DATASET_STAT_DONE;
        }
    }
}

//---DataSet::setIndices------------------------------------------------------
// Input :
// Output:
// sets the variable Data::index according to the printing order generated
// by printSet - uses the mechanism as printSet e.g. countUncompressed

void DataSet::setIndices()
{
    int i;
    int co = refArray->getSize();
    int currIndex=0; 
    Data *data;
 
    for (i=0; i<co; i++)
    {
        data=getIndex(i);
        data->status &= ~DATASET_STAT_DONE;
    }

    for (i=0; i<co; i++)
    {
        data=getIndex(i);
        if ((data->status & DATASET_STAT_DONE)==0 || (i<countUncompressed))
        {
            data->index = currIndex;
            currIndex++;
            data->status |= DATASET_STAT_DONE;
        }
    }
    count=currIndex;
}

//----------------------------------------------------------------------------
