// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       enum
//  TYPE:       c++ header
//  PROJECT:    lodestar
//  CONTENT:    id and string definitions for Node, Field and Value classes
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================
//  MANUAL:
//
//
//
// ===========================================================================

#ifndef _ENUM_H_
#define _ENUM_H_

//----------------------------------------------------------------------------
//   Node-Identifier
//----------------------------------------------------------------------------

enum NodeIdentifier {
    NID_ASCIITEXT, NID_CONE, NID_COORDINATE3, NID_CUBE, NID_CYLINDER,
    NID_DIRECTIONALLIGHT, NID_FONTSTYLE, NID_GROUP, NID_INDEXEDFACESET,
    NID_INDEXEDLINESET, NID_INFO, NID_LOD, NID_MATERIAL,
    NID_MATERIALBINDING, NID_LEVELOFDETAIL, NID_MATRIXTRANSFORM,
    NID_NORMAL, NID_NORMALBINDING, NID_ORTHOGRAPHICCAMERA,
    NID_PERSPECTIVECAMERA, NID_POINTLIGHT, NID_POINTSET, NID_ROTATION,
    NID_SCALE, NID_SEPARATOR, NID_SHAPEHINTS, NID_SPHERE, NID_SPOTLIGHT,
    NID_SWITCH, NID_TEXTURE2, NID_TEXTURE2TRANSFORM,
    NID_TEXTURECOORDINATE2, NID_TRANSFORM, NID_TRANSFORMSEPARATOR,
    NID_TRANSLATION, NID_WWWANCHOR, NID_WWWINLINE, 
    NID_UNDEFINED
};

extern const char *nodeIdString[NID_UNDEFINED+1];

//----------------------------------------------------------------------------
//   Field-Identifier
//----------------------------------------------------------------------------

enum FieldIdentifier {
    FID_PARTS, FID_BOTTOMRADIUS, FID_HEIGHT, FID_POINT, FID_WIDTH,
    FID_DEPTH, FID_STRING, FID_SPACING, FID_JUSTIFICATION, FID_RADIUS,
    FID_ON, FID_INTENSITY, FID_COLOR, FID_DIRECTION, FID_SIZE,
    FID_FAMILY, FID_STYLE, FID_COORDINDEX, FID_MATERIALINDEX,
    FID_NORMALINDEX, FID_TEXTURECOORDINDEX, FID_RANGE, FID_SCREENAREA,
    FID_TRANSLATION, FID_CENTER, FID_AMBIENTCOLOR, FID_DIFFUSECOLOR,
    FID_SPECULARCOLOR, FID_EMISSIVECOLOR, FID_SHININESS,
    FID_TRANSPARENCY, FID_VALUE, FID_MATRIX, FID_VECTOR, FID_POSITION,
    FID_ORIENTATION, FID_FOCALDISTANCE, FID_HEIGHTANGLE, FID_LOCATION,
    FID_STARTINDEX, FID_NUMPOINTS, FID_ROTATION, FID_SCALEFACTOR,
    FID_RENDERCULLING, FID_VERTEXORDERING, FID_SHAPETYPE, FID_FACETYPE,
    FID_CREASEANGLE, FID_DROPOFFRATE, FID_CUTOFFANGLE, FID_WHICHCHILD,
    FID_FILENAME, FID_IMAGE, FID_WRAPS, FID_WRAPT, FID_SCALEORIENTATION,
    FID_NAME, FID_DESCRIPTION, FID_MAP, FID_BBOXSIZE, FID_BBOXCENTER,
    FID_UNDEFINED
};

extern const char *fieldIdString[FID_UNDEFINED+1];

//----------------------------------------------------------------------------
//   Mnemonics for Values of Fields
//----------------------------------------------------------------------------

//--- parts of cones and cylinder --------------------------------------------

#define VRML_SIDES   1
#define VRML_BOTTOM  2
#define VRML_TOP     4
#define VRML_ALL    (VRML_SIDES & VRML_BOTTOM & VRML_TOP)

//--- font styles ------------------------------------------------------------

#define VRML_NONE    8
#define VRML_BOLD   16
#define VRML_ITALIC 32

//--- text justification -----------------------------------------------------

#define VRML_LEFT    0
#define VRML_CENTER  1
#define VRML_RIGHT   2

//--- font family ------------------------------------------------------------

#define VRML_SERIF      0
#define VRML_SANS       1
#define VRML_TYPEWRITER 2

//--- bindings ---------------------------------------------------------------

#define VRML_DEFAULT            0
#define VRML_OVERALL            1
#define VRML_PER_PART           2
#define VRML_PER_PART_INDEXED   3
#define VRML_PER_FACE           4
#define VRML_PER_FACE_INDEXED   5
#define VRML_PER_VERTEX         6
#define VRML_PER_VERTEX_INDEXED 7

//--- culling ----------------------------------------------------------------

#define VRML_ON   0
#define VRML_OFF  1
#define VRML_AUTO 2

//--- vertex ordering --------------------------------------------------------

#define VRML_UNKNOWN_ORDERING 0
#define VRML_CLOCKWISE        1
#define VRML_COUNTERCLOCKWISE 2

//--- shape type -------------------------------------------------------------

#define VRML_UNKNOWN_SHAPE_TYPE 0
#define VRML_SOLID              1

//--- face type --------------------------------------------------------------

#define VRML_UNKNOWN_FACE_TYPE 0
#define VRML_CONVEX            1

//--- wrap -------------------------------------------------------------------

#define VRML_REPEAT 0
#define VRML_CLAMP  1

//----------------------------------------------------------------------------

#define VRML_POINT VRML_NONE+1

//--- map --------------------------------------------------------------------

#endif
