// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       fields.h
//  TYPE:       c++ header
//  PROJECT:    lodestar
//  CONTENT:    class Value and all classes derived from class Value
//              class Field
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================

#include "fields.h"
#include "enum.h"
#include "global.h"
#include <string.h>
#include <assert.h>

//-----------------------------------------------------------------------------
//   class Value
//-----------------------------------------------------------------------------

//---Value::Value--------------------------------------------------------------
// Input : 
// Output: 

Value::Value()
{
    next=NULL;
}

//---Value::~Value-------------------------------------------------------------
// Input : 
// Output: 

Value::~Value()
{
}

//---Value::print--------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:
// abstract function

void Value::print(FILE * fd)
{
}

//---Value::printLn------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
//         int indent     : indent for the next line
// Output:
// prints a carrige return into the output file

void Value::printLn(FILE * fd,int indent)
{
    fprintf(fd,"\n");
    printIndent(fd,indent);
}

//-----------------------------------------------------------------------------
// FBitMask
//-----------------------------------------------------------------------------

//---FBitMask::FBitMask--------------------------------------------------------
// Input : int anumber
// Output: 

FBitMask::FBitMask(int anumber) : Value()
{
    number=anumber;
    type=FTnothing;
}

//---FBitMask::print-----------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:
// If type is FTparts then print "ALL","SIDES"

void FBitMask::print(FILE * fd)
{
    switch (type)
    {
        // Cylinder,Cone ------------------------------------------------------
        case FTparts:
            if (number==(VRML_SIDES&VRML_BOTTOM&VRML_TOP)) fprintf(fd,"ALL");
            else
            {
                int multi=number!=VRML_SIDES
                    &&number!=VRML_BOTTOM&&number!=VRML_TOP;
                if (multi) fprintf(fd,"(");
                if (number&VRML_SIDES)
                {
                    fprintf(fd,"SIDES");
                    if (multi) fprintf(fd,"|");
                }
                if (number&VRML_BOTTOM)
                {
                    fprintf(fd,"BOTTOM");
                    if (number&VRML_TOP) fprintf(fd,"|");
                }
                if (number&VRML_TOP) fprintf(fd,"TOP");
                if (multi) fprintf(fd,")");
            }
            break;

        default: assert(0);
            
    }
}

//-----------------------------------------------------------------------------
// FBool
//-----------------------------------------------------------------------------

//---FBool::FBool--------------------------------------------------------------
// Input : int anumber      : TRUE or FALSE
// Output:

FBool::FBool(int anumber) : Value()
{
    number=anumber;
}

//---FBool::print--------------------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FBool::print(FILE * fd)
{
    if (number) fprintf(fd,"TRUE");
    else fprintf(fd,"FALSE");
}

//-----------------------------------------------------------------------------
// FEnum
//-----------------------------------------------------------------------------

//---FEnum--------------------------------------------------------------------------
// Input : int anumber       init
// Output:

FEnum::FEnum(int anumber) : Value()
{
    number=anumber;
    type=FTnothing;
}

//-----------------------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:
// print string constant depending on type and number

void FEnum::print(FILE * fd)
{
    switch (type) 
    {
        // Font ---------------------------------------------------------------
        case FTfamily:    
            switch (number)
            {        
                case VRML_SERIF:      fprintf(fd,"SERIF");               break;
                case VRML_SANS:       fprintf(fd,"SANS");                break;
                case VRML_TYPEWRITER: fprintf(fd,"TYPEWRITER");          break;
                default: assert(0);
            }
            break;
        // ShapeHints ---------------------------------------------------------
        case FTvertexOrdering:    
            switch (number)
            {        
              case VRML_UNKNOWN_ORDERING: fprintf(fd,"UNKNOWN_ORDERING");break;
              case VRML_CLOCKWISE:        fprintf(fd,"CLOCKWISE");       break;
              case VRML_COUNTERCLOCKWISE: fprintf(fd,"COUNTERCLOCKWISE");break;
              default: assert(0);
            }
            break;
        case FTshapeType:
            switch (number)
            {        
                case VRML_UNKNOWN_SHAPE_TYPE: 
                    fprintf(fd,"UNKNOWN_SHAPE_TYPE");
                    break;
                case VRML_SOLID:              
                    fprintf(fd,"SOLID");                                 
                    break;
                default: assert(0);
            }
            break;
        case FTfaceType:
            switch (number)
            {        
                case VRML_UNKNOWN_FACE_TYPE: 
                    fprintf(fd,"UNKNOWN_FACE_TYPE");  
                    break;
                case VRML_CONVEX: 
                    fprintf(fd,"CONVEX");             
                    break;
                default: assert(0);
            }
            break;
        // Texture2 -----------------------------------------------------------
        case FTwrapS:
        case FTwrapT:
            switch (number)
            {        
                case VRML_REPEAT: fprintf(fd,"REPEAT");                  break;
                case VRML_CLAMP:  fprintf(fd,"CLAMP");                   break;
                default: assert(0);
            }
            break;
        // Separator ----------------------------------------------------------
        case FTrenderCulling:
            switch (number)
            {        
                case VRML_ON:   fprintf(fd,"ON");                        break;
                case VRML_OFF:  fprintf(fd,"OFF");                       break;
                case VRML_AUTO: fprintf(fd,"AUTO");                      break;
                default: assert(0);
            }
            break;
        // NormalBinding, MaterialBinding -------------------------------------
        case FTvalue:
            switch (number)
            {        
                 case VRML_DEFAULT:            
                    fprintf(fd,"DEFAULT");           
                    break;
                case VRML_OVERALL:            
                    fprintf(fd,"OVERALL");           
                    break;
                case VRML_PER_PART:           
                    fprintf(fd,"PER_PART");          
                    break;
                case VRML_PER_PART_INDEXED:   
                    fprintf(fd,"PER_PART_INDEXED");  
                    break;
                case VRML_PER_FACE:           
                    fprintf(fd,"PER_FACE");          
                    break;
                case VRML_PER_FACE_INDEXED:   
                    fprintf(fd,"PER_FACE_INDEXED");  
                    break;
                case VRML_PER_VERTEX:         
                    fprintf(fd,"PER_VERTEX");        
                    break;
                case VRML_PER_VERTEX_INDEXED: 
                    fprintf(fd,"PER_VERTEX_INDEXED");
                    break;
                default: assert(0);
            }
            break;
        // AsciiText -------------------------------------
        case FTjustification:
            switch (number)
            {        
                 case VRML_LEFT:   fprintf(fd,"LEFT");    break;
                 case VRML_CENTER: fprintf(fd,"CENTER");  break;
                 case VRML_RIGHT:  fprintf(fd,"RIGHT");   break;
                 default: assert(0);
            }
            break;
        // WWWAnchor -------------------------------------
        case FTmap:
            switch (number)
            {        
                 case VRML_NONE:  fprintf(fd,"NONE");   break;
                 case VRML_POINT: fprintf(fd,"POINT");  break;
                 default: assert(0);
            }
            break;


        default: assert(0);
    }
}

//-----------------------------------------------------------------------------
// FFloat
//-----------------------------------------------------------------------------

//---FFloat::FFloat------------------------------------------------------------
// Input : float anumber     : init
// Output:

FFloat::FFloat(float anumber) : Value()
{
    number=anumber;
}

//---FFloat::print-------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FFloat::print(FILE * fd)
{
    printFloat(fd,number);
    global_cpl++;
}

//---FFloat::printLn-----------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
//         int indent     : see Value::printLn
// Output:
// With global_floatcpl you can specify the float count per line
// global_floatcpl will be set as program parameter

void FFloat::printLn(FILE * fd,int indent)
{
    if (global_cpl>=global_floatcpl)
    {
        Value::printLn(fd,indent);
        global_cpl=0;
    }
}

//-----------------------------------------------------------------------------
// FImage
//-----------------------------------------------------------------------------

//---FImage::FImage------------------------------------------------------------
// Input : int anumber     : init
//         Value * anext   : pointer to the next FImage object
// Output:
// With the next pointer you can make a list
// First element : width
// Second element: height
// Third elemen  : number of color items
// Rest          : color items

FImage::FImage(int anumber,Value * anext) : Value()
{
    number=anumber;
    next=anext;
}

//---FImage::print-------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FImage::print(FILE * fd)
{
    fprintf(fd,"%d ",number);
}

//-----------------------------------------------------------------------------
// FLong
//-----------------------------------------------------------------------------

//---FLong::FLong--------------------------------------------------------------
// Input : int anumber     : init
// Output:

FLong::FLong(int anumber) : Value()
{
    number=anumber;
}

//---FLong::print--------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FLong::print(FILE * fd)
{
    fprintf(fd,"%d",number);
    global_cpl++;
}

//---FLong::printLn------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
//         int indent     : see Value::printLn
// Output:
// With global_longcpl you can specify the long count per line
// global_longcpl will be set as program parameter

void FLong::printLn(FILE * fd,int indent)
{
    if (global_cpl>=global_longcpl)
    {
        Value::printLn(fd,indent);
        global_cpl=0;
    }
}

//-----------------------------------------------------------------------------
// FMatrix
//-----------------------------------------------------------------------------

//---FMatrix::FMatrix----------------------------------------------------------
// Input : anumber11 to anumber44 : init
// Output:

FMatrix::FMatrix(float anumber11,float anumber12,float anumber13,float anumber14,
                 float anumber21,float anumber22,float anumber23,float anumber24,
                 float anumber31,float anumber32,float anumber33,float anumber34,
                 float anumber41,float anumber42,float anumber43,float anumber44) :
        Value()
{
    number11=anumber11;
    number12=anumber12;
    number13=anumber13;
    number14=anumber14;
    number21=anumber21;
    number22=anumber22;
    number23=anumber23;
    number24=anumber24;
    number31=anumber31;
    number32=anumber32;
    number33=anumber33;
    number34=anumber34;
    number41=anumber41;
    number42=anumber42;
    number43=anumber43;
    number44=anumber44;
}

//---FMatrix::print------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FMatrix::print(FILE * fd)
{
    printFloat(fd,number11); fprintf(fd," ");
    printFloat(fd,number12); fprintf(fd," ");
    printFloat(fd,number13); fprintf(fd," ");
    printFloat(fd,number14); fprintf(fd," ");
    printFloat(fd,number21); fprintf(fd," ");
    printFloat(fd,number22); fprintf(fd," ");
    printFloat(fd,number23); fprintf(fd," ");
    printFloat(fd,number24); fprintf(fd," ");
    printFloat(fd,number31); fprintf(fd," ");
    printFloat(fd,number32); fprintf(fd," ");
    printFloat(fd,number33); fprintf(fd," ");
    printFloat(fd,number34); fprintf(fd," ");
    printFloat(fd,number41); fprintf(fd," ");
    printFloat(fd,number42); fprintf(fd," ");
    printFloat(fd,number43); fprintf(fd," ");
    printFloat(fd,number44); fprintf(fd,"\n");
}

//-----------------------------------------------------------------------------
// FRotation
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Input : float ax,ay,az : init
//         float aangle   : init

FRotation::FRotation(float ax,float ay,float az, float aangle) : Value()
{
    x=ax;
    y=ay;
    z=az;
    angle=aangle;
}

//---FRotation::print----------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FRotation::print(FILE * fd)
{
    printFloat(fd,x); fprintf(fd," ");
    printFloat(fd,y); fprintf(fd," ");
    printFloat(fd,z); fprintf(fd," ");
    printFloat(fd,angle);
}

//-----------------------------------------------------------------------------
// FString
//-----------------------------------------------------------------------------

//---FString::FString----------------------------------------------------------
// Input : char * astring   : init
// Output: 

FString::FString(char * astring) : Value()
{
    string=astring;
}

//---FString::~FString---------------------------------------------------------
// Input : 
// Output:
// delete the char array

FString::~FString()
{
    delete string;
}

//---FString::print------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FString::print(FILE * fd)
{
    fprintf(fd,"\"%s\"",string);
}

//-----------------------------------------------------------------------------
// FVec2f
//-----------------------------------------------------------------------------

//---FVec2f::FVec2f------------------------------------------------------------
// Input : float anumber1 : init
//         float anumber2 : init
// Output: 

FVec2f::FVec2f(float anumber1,float anumber2) : Value()
{
    x=anumber1;
    y=anumber2;
}

//---FVec2f::print-------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FVec2f::print(FILE * fd)
{
    printFloat(fd,x); fprintf(fd," ");
    printFloat(fd,y);
    global_cpl+=2;
}

//-----------------------------------------------------------------------------
// FVec3f
//-----------------------------------------------------------------------------

//---FVec3f::FVec3f------------------------------------------------------------
// Input : float anumber1  : init
//         float anumber2  : init
//         float anumber3  : init

FVec3f::FVec3f(float anumber1,float anumber2,float anumber3) : Value()
{
    x=anumber1;
    y=anumber2;
    z=anumber3;
}

//---FVec3f::print-------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
// Output:

void FVec3f::print(FILE * fd)
{
    printFloat(fd,x); fprintf(fd," ");
    printFloat(fd,y); fprintf(fd," ");
    printFloat(fd,z);
    global_cpl+=3;
}

//-----------------------------------------------------------------------------
// Field
//-----------------------------------------------------------------------------

//---Field::Field--------------------------------------------------------------
// Input : int aid         : init
//         Value * avalue  : init
// Output: 
// There is only one value pointer in the class Field, but value can point to
// a next Value object (and so on).

Field::Field(int aid, Value * avalue)
{
    id=aid;
    value=avalue;
    next=NULL;
}

//---Field::~Field-------------------------------------------------------------
// Input :
// Output:

Field::~Field()
{
    Value *vh;
    while (value!=NULL)
    {
       vh=value;
       value=value->next;
       delete vh; 
    }
}

//---Field::print--------------------------------------------------------------
// Input : FILE * fd      : lod VRML file descriptor
//         int indent     : indent for field id and values
// Output:
// Prints field id, brackets and call print function of Value objects

void Field::print(FILE * fd, int indent)
{
    printIndent(fd,indent);
    fprintf(fd,"%s ",fieldIdString[id]);

    int indent2 = indent+strlen(fieldIdString[id]);

    if (value==NULL || value->next!=NULL) 
    {
        indent2+=2;
        fprintf(fd,"[ ");
    }

    if (value)
    {
        value->print(fd);

        Value *vh = value->next;
        while (vh!=NULL)
        {
            fprintf(fd,",");   
            vh->printLn(fd,indent2);
            vh->print(fd);
            vh=vh->next;
        }
    }

    if (value==NULL || value->next!=NULL) fprintf(fd," ]");

    fprintf(fd,"\n");
}

//-----------------------------------------------------------------------------

