// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       global
//  TYPE:       c++ header
//  PROJECT:    lodestar
//  CONTENT:    global variables and functions and some basic classes
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================
//  MANUAL:
//
//
//
// ===========================================================================

#ifndef _GLOBAL_H_
#define _GLOBAL_H_

#include <stdio.h>
#include <math.h>

//----------------------------------------------------------------------------
// global variables - used by function printIdent and many print-methods
//----------------------------------------------------------------------------

extern int global_indent;   // count of blanks of indention for subnodes,fields  
extern int global_tab;      // count of blanks of a tab for optimal fill
extern int global_noindent; // suppresses the indention 
extern int global_floatcpl; // count of floatingpoint-numbers per line
extern int global_longcpl;  // count of integer-numbers per line
extern int global_cpl;      // count of numbers printed in this line so far

//----------------------------------------------------------------------------
//   simple 3D vector class
//----------------------------------------------------------------------------

class Matrix; //forward definition

class Vec
{
public:

    float x;
    float y;
    float z;

    Vec();
    Vec(float ax, float ay, float az);
    float getLength();
    int isNullVector();

    Vec &operator /=(float f);

    friend Vec operator +(const Vec &v1, const Vec &v2);
    friend Vec operator -(const Vec &v1, const Vec &v2);

    friend Vec mult3x3(const Vec &v,        //multiplies Vec with
                       const Matrix &m);    //the 3x3 matrix e11 to e33
};

//----------------------------------------------------------------------------
//   4x4 Matrix class
//----------------------------------------------------------------------------

class Matrix
{
public:

    float e[4][4];

    Matrix();
    Matrix(float e11,float e12,float e13,float e14,
           float e21,float e22,float e23,float e24,
           float e31,float e32,float e33,float e34,
           float e41,float e42,float e43,float e44);

    friend Matrix operator *(const Matrix &m1, const Matrix &m2);

    friend Vec mult3x3(const Vec &v,        //multiplies Vec with
                       const Matrix &m);    //the 3x3 matrix e11 to e33
};

//----------------------------------------------------------------------------
//   class for calculating the bounds of some points in 3D
//----------------------------------------------------------------------------

class Bounds
{
public:

    float xmin;
    float xmax;
    float ymin;
    float ymax;
    float zmin;
    float zmax;

    Bounds() {}
    Bounds(float axmin, float aymin, float azmin, float axmax, float aymax, float azmax);
    void setTo(float axmin, float aymin, float azmin, float axmax, float aymax, float azmax);
    void enlargeIfOutside(float x, float y, float z);
    void makeCubic();
    float getMaxElongation();
};

//----------------------------------------------------------------------------
//   writes tabs and blanks to the file determined by fd
//   if the global variable global_noindent != 0 then no action is performed
//   e.g. indent = 10
//        global_noindent = 0;
//        global_tab = 8;
//        the function writes 1 tab and 2 blanks to the file determined by fd 
//----------------------------------------------------------------------------

void printIndent(FILE *fd, int indent);

//----------------------------------------------------------------------------
//   converts value into a string, removes trailing zeros and writes the
//   string to the file determined by fd
//----------------------------------------------------------------------------

void printFloat(FILE *fd, double value);

//----------------------------------------------------------------------------

#endif

