// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       ident
//  TYPE:       c++ code
//  PROJECT:    lodestar
//  CONTENT:    
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================

#include "nodes.h"
#include <string.h>

//----------------------------------------------------------------------------
//   class Ident
//----------------------------------------------------------------------------

//---Ident::Ident-------------------------------------------------------------
// Input : Node *anode       : init
// Output:

Ident::Ident(Node *anode)
{
    node = anode;
    left = NULL;
    right= NULL;
}

//----------------------------------------------------------------------------
//   class IdentSet
//----------------------------------------------------------------------------

//---IdentSet::IdentSet-------------------------------------------------------
// Input :
// Output:

IdentSet::IdentSet()
{
    root=NULL;
}

//---IdentSet::~IdentSet------------------------------------------------------
// Input :
// Output:

IdentSet::~IdentSet()
{
    deleteTree(root);
}

//---IdentSet::insert---------------------------------------------------------
// Input : Ident *ident        : new ident for insert
// Output:

void IdentSet::insert(Ident *ident)
{
    insertTree(ident, &root);    
}

//---IdentSet::search---------------------------------------------------------
// Input : const char *name     : search ident with name char * name
// Output: Ident *              : ident with name name

Ident *IdentSet::search(const char *name)
{
    return searchTree(name, root);
}

//---IdentSet::insertTree-----------------------------------------------------
// Input : Ident * ident        : new ident
//         Ident * root         : root of tree
// Output:
// insert new Ident object on the place
// The left side contains the object with the smaller and the right side the 
// object with the greater string.

void IdentSet::insertTree(Ident *ident, Ident **root)
{
    if ((*root)==NULL) 
    {
        *root = ident;
        return;
    }

    int result = strcmp(ident->node->nodespec, (*root)->node->nodespec);

    if (result<0) insertTree(ident,&((*root)->left));
             else insertTree(ident,&((*root)->right));
}

//---IdentSet::searchTree-----------------------------------------------------
// Input : const char *name    : search ident with name name
//         Ident * root        : root of tree
// Output: Ident *             : ident with name name
// find the last identifer with the name
 
Ident *IdentSet::searchTree(const char *name, Ident *root)
{
    if (root==NULL) return NULL;

    int result = strcmp(name, root->node->nodespec);

    if (result<0) return searchTree(name,root->left);
    if (result>0) return searchTree(name,root->right);
    
    while (root->right && strcmp(name, root->right->node->nodespec)==0)
        root = root->right;
    return root;
}

//---IdentSet::deleteTree-----------------------------------------------------
// Input : Ident * root        : delete tree with root root
// Output:

void IdentSet::deleteTree(Ident *root)
{
    if (root == NULL) return;
    deleteTree(root->left);
    deleteTree(root->right);
    delete root;
}

//----------------------------------------------------------------------------















