%{
// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       lex
//  TYPE:       lex scanner
//  PROJECT:    lodestar
//  CONTENT:    lex scanner for VRML files
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//              hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//  based on a lex scanner written by Emmanuel Frcon and Olof Hagsand
//
// ===========================================================================
//  MANUAL:
//
// ===========================================================================

#include <string.h>
#include "fields.h"     /* must be before yacc.h */
#include "nodes.h"	/* must be before yacc.h */
#include "yacc.h"
#include "enum.h"

/*==========================================================================
  variables
  ==========================================================================*/

int lex_line = 1;
int lex_idline = 1;

static int unknowncount=0;

%}

/*****************************************************************************/
/* lexical definitions */
/*****************************************************************************/

%s start_use_and_def start_fields start_fields2 start_str start_unknown_node

INSTANCE   [^\x00-\x20\x7f-\xff\{\}\'\"\#\\\+\.0-9][^\x00-\x20\x7f-\xff\{\}\'\"\#\\\+\.]*
ID         [A-Z][^\x00-\x20\x7f-\xff\{\}\[\]\'\"\#\\\+\.\&]*

%%


<start_use_and_def>[ \t]+	;
<start_use_and_def>\n		lex_line++;
<start_use_and_def>{INSTANCE}	{ yylval.string = strdup(yytext);
				  BEGIN(INITIAL);
				  return VRMLWORD; }


<start_fields>[ \t]+	       	;
<start_fields>\n		{ lex_line++; }
<start_fields>"["		{ yymore(); BEGIN(start_fields2); }
<start_fields2>[^\]\n]+		{ yymore(); }
<start_fields2>\n		{ yymore(); lex_line++; }
<start_fields2>"]"		{ yylval.string = strdup(yytext);
				  BEGIN(INITIAL);
				  return FIELDSPEC; }


<start_str>[^\n\"\\]+		{ yymore(); }
<start_str>\\\n			{ yymore(); lex_line++; }
<start_str>\\.			{ yymore(); }
<start_str>\n			{ yymore(); lex_line++; }
<start_str>\"			{ int l=strlen(yytext);
				  yylval.string = strncpy((char *)malloc(l),yytext,l-1);
				  yylval.string[l-1]='\0';
				  BEGIN(INITIAL);
				  return VRMLSTRING; }

<start_unknown_node>"{"		{ yymore(); unknowncount++; }
<start_unknown_node>"}"		{ unknowncount--;
                                  yymore();
				  if (unknowncount==0)
                                  {
				    BEGIN(INITIAL);
                                    yylval.string = strdup(yytext);
                                    return UNKNOWNNODELIST;
				  }
				}
<start_unknown_node>[^{}\n]+	{ yymore(); }
<start_unknown_node>\n		{ yymore(); lex_line++; }


DEF				{ BEGIN(start_use_and_def); return DEF; }
USE				{ yylval.line=lex_line; BEGIN(start_use_and_def); return USE; }

AsciiText			{ yylval.line=lex_line; return ASCIITEXT; }
Cone				{ yylval.line=lex_line; return CONE; }
Coordinate3			{ yylval.line=lex_line; return COORDINATE3; }
Cube				{ yylval.line=lex_line; return CUBE; }
Cylinder			{ yylval.line=lex_line; return CYLINDER; }
DirectionalLight		{ yylval.line=lex_line; return DIRECTIONALLIGHT; }
FontStyle			{ yylval.line=lex_line; return FONTSTYLE; }
Group				{ yylval.line=lex_line; return GROUP; }
IndexedFaceSet			{ yylval.line=lex_line; return INDEXEDFACESET; }
IndexedLineSet			{ yylval.line=lex_line; return INDEXEDLINESET; }
Info				{ yylval.line=lex_line; return INFO; }
LOD				{ yylval.line=lex_line; return LOD; }
LevelOfDetail			{ yylval.line=lex_line; return LEVELOFDETAIL; }
Material			{ yylval.line=lex_line; return MATERIAL; }
MaterialBinding			{ yylval.line=lex_line; return MATERIALBINDING; }
MatrixTransform			{ yylval.line=lex_line; return MATRIXTRANSFORM; }
Normal				{ yylval.line=lex_line; return NORM; }
NormalBinding			{ yylval.line=lex_line; return NORMALBINDING; }
OrthographicCamera		{ yylval.line=lex_line; return ORTHOGRAPHICCAMERA; }
PerspectiveCamera		{ yylval.line=lex_line; return PERSPECTIVECAMERA; }
PointLight			{ yylval.line=lex_line; return POINTLIGHT; }
PointSet			{ yylval.line=lex_line; return POINTSET; }
Rotation			{ yylval.line=lex_line; return ROTATION; }
Scale				{ yylval.line=lex_line; return SCALE; }
Separator			{ yylval.line=lex_line; return SEPARATOR; }
ShapeHints			{ yylval.line=lex_line; return SHAPEHINTS; }
Sphere				{ yylval.line=lex_line; return SPHERE; }
SpotLight			{ yylval.line=lex_line; return SPOTLIGHT; }
Switch				{ yylval.line=lex_line; return SWITCH; }
Texture2			{ yylval.line=lex_line; return TEXTURE2; }
Texture2Transform		{ yylval.line=lex_line; return TEXTURE2TRANSFORM; }
TextureCoordinate2		{ yylval.line=lex_line; return TEXTURECOORDINATE2; }
Transform			{ yylval.line=lex_line; return TRANSFORM; }
TransformSeparator		{ yylval.line=lex_line; return TRANSFORMSEPARATOR; }
Translation			{ yylval.line=lex_line; return TRANSLATION; }
WWWAnchor			{ yylval.line=lex_line; return WWWANCHOR; }
WWWInline			{ yylval.line=lex_line; return WWWINLINE; }


parts				{ return PARTS; }
bottomRadius			{ return BOTTOMRADIUS; }
height				{ return HEIGHT; }
point				{ return POINT; }
width				{ return WIDTH; }
depth				{ return DEPTH; }
string				{ return STRING; }
spacing				{ return SPACING; }
justification			{ return JUSTIFICATION; }
radius				{ return RADIUS; }
on				{ return ON; }
intensity			{ return INTENSITY; }
color				{ return COLOR; }
direction			{ return DIRECTION; }
size				{ return SIZE; }
family				{ return FAMILY; }
style				{ return STYLE; }
coordIndex			{ return COORDINDEX; }
materialIndex			{ return MATERIALINDEX; }
normalIndex			{ return NORMALINDEX; }
textureCoordIndex		{ return TEXTURECOORDINDEX; }
range				{ return RANGE; }
center				{ return CENTER; }
screenArea			{ return SCREENAREA; }
ambientColor			{ return AMBIENTCOLOR; }
diffuseColor			{ return DIFFUSECOLOR; }
specularColor			{ return SPECULARCOLOR; }
emissiveColor			{ return EMISSIVECOLOR; }
shininess			{ return SHININESS; }
transparency			{ return TRANSPARENCY; }
value				{ return VALUE; }
matrix				{ return MATRIX; }
vector				{ return VECTOR; }
position			{ return POSITION; }
orientation			{ return ORIENTATION; }
focalDistance			{ return FOCALDISTANCE; }
heightAngle			{ return HEIGHTANGLE; }
location			{ return LOCATION; }
startIndex			{ return STARTINDEX; }
numPoints			{ return NUMPOINTS; }
rotation			{ return ROTATION_F; }
scaleFactor			{ return SCALEFACTOR; }
renderCulling			{ return RENDERCULLING; }
vertexOrdering			{ return VERTEXORDERING; }
shapeType			{ return SHAPETYPE; }
faceType			{ return FACETYPE; }
creaseAngle			{ return CREASEANGLE; }
dropOffRate			{ return DROPOFFRATE; }
cutOffAngle			{ return CUTOFFANGLE; }
whichChild			{ return WHICHCHILD; }
filename			{ return FILENAME; }
image				{ return IMAGE; }
wrapS				{ return WRAPS; }
wrapT				{ return WRAPT; }
scaleOrientation		{ return SCALEORIENTATION; }
translation			{ return TRANSLATION_F; }
name				{ return NAME; }
description			{ return DESCRIPTION; }
map				{ return MAP; }
bboxSize			{ return BBOXSIZE; }
bboxCenter			{ return BBOXCENTER; }


LEFT				{ yylval.intval = (int)VRML_LEFT; 
				  return NUMBER; }
CENTER				{ yylval.intval = (int)VRML_CENTER;
				  return NUMBER; }
RIGHT				{ yylval.intval = (int)VRML_RIGHT;
				  return NUMBER; }

SIDES				{ yylval.intval = (int)VRML_SIDES;
				  return NUMBER; }
BOTTOM				{ yylval.intval = (int)VRML_BOTTOM;
				  return NUMBER; }
TOP				{ yylval.intval = (int)VRML_TOP;
				  return NUMBER; }
ALL				{ yylval.intval = (int)VRML_ALL;
				  return NUMBER; }

TRUE				{ yylval.intval = 1; return NUMBER; }
FALSE				{ yylval.intval = 0; return NUMBER; }

SERIF				{ yylval.intval = (int)VRML_SERIF; 
				  return NUMBER; }
SANS				{ yylval.intval = (int)VRML_SANS;
				  return NUMBER; }
TYPEWRITER			{ yylval.intval = (int)VRML_TYPEWRITER;
				  return NUMBER; }

NONE				{ yylval.intval = (int)VRML_NONE;
				  return NUMBER; }
BOLD				{ yylval.intval = (int)VRML_BOLD;
				  return NUMBER; }
ITALIC				{ yylval.intval = (int)VRML_ITALIC;
				  return NUMBER; }

DEFAULT				{ yylval.intval = (int)VRML_DEFAULT;
				  return NUMBER; }
OVERALL				{ yylval.intval = (int)VRML_OVERALL;
				  return NUMBER; }
PER_PART			{ yylval.intval = (int)VRML_PER_PART;
				  return NUMBER; }
PER_PART_INDEXED		{ yylval.intval = (int)VRML_PER_PART_INDEXED;
				  return NUMBER; }
PER_FACE			{ yylval.intval = (int)VRML_PER_FACE;
				  return NUMBER; }
PER_FACE_INDEXED		{ yylval.intval = (int)VRML_PER_FACE_INDEXED;
				  return NUMBER; }
PER_VERTEX			{ yylval.intval = (int)VRML_PER_VERTEX;
				  return NUMBER; }
PER_VERTEX_INDEXED		{ yylval.intval = (int)VRML_PER_VERTEX_INDEXED;
				  return NUMBER; }

ON				{ yylval.intval = (int)VRML_ON;
				  return NUMBER; }
OFF				{ yylval.intval = (int)VRML_OFF;
				  return NUMBER; }
AUTO				{ yylval.intval = (int)VRML_AUTO;
				  return NUMBER; }

UNKNOWN_ORDERING		{ yylval.intval = (int)VRML_UNKNOWN_ORDERING;
				  return NUMBER; }
CLOCKWISE			{ yylval.intval = (int)VRML_CLOCKWISE;
				  return NUMBER; }
COUNTERCLOCKWISE		{ yylval.intval = (int)VRML_COUNTERCLOCKWISE;
				  return NUMBER; }

UNKNOWN_SHAPE_TYPE		{ yylval.intval = (int)VRML_UNKNOWN_SHAPE_TYPE;
				  return NUMBER; }
SOLID				{ yylval.intval = (int)VRML_SOLID;
				  return NUMBER; }

UNKNOWN_FACE_TYPE		{ yylval.intval = (int)VRML_UNKNOWN_FACE_TYPE;
				  return NUMBER; }
CONVEX				{ yylval.intval = (int)VRML_CONVEX;
				  return NUMBER; }

REPEAT				{ yylval.intval = (int)VRML_REPEAT;
				  return NUMBER; }
CLAMP				{ yylval.intval = (int)VRML_CLAMP;
				  return NUMBER; }

POINT				{ yylval.intval = (int)VRML_POINT;
				  return NUMBER; }

fields				{ BEGIN(start_fields); return FIELDS; } 


0				{ yylval.intval = 0; return NUMBER;}
-?[1-9][0-9]*			{ yylval.intval = atoi(yytext); return NUMBER;}
0[1-7][0-7]*			{ sscanf((const char *)yytext,
					 "%o", &yylval.intval);
				  return NUMBER; }
-?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?	{ yylval.floatval = atof (yytext);
					  return FLOAT;}
0x[0-9A-Fa-f]+			{ sscanf((const char *)yytext,
				  "0x%x", &yylval.intval);
				  return NUMBER; }

[\#].*\n			lex_line++; /* Skip rest of line. */

\"				{ BEGIN(start_str); }

[ \t]+				;
\n				lex_line++;

{ID}				{
				  lex_idline = lex_line;
                                  yylval.string = strdup(yytext);
				  BEGIN(start_unknown_node);
				  return UNKNOWNNODE;
				}

[|]				{ return *yytext; }
","				{ return ','; }
"}"				{ return '}'; }
"{"				{ return '{'; }
"["				{ return '['; }
"]"				{ return ']'; }
.				{ fprintf(stderr,"warning: %d: skipping illegal input >%s<\n",lex_line,yytext); }

%%

void lex_setInput(FILE *f)
{
#ifdef FLEX_SCANNER
  static int firstcall = 1;
#endif

  lex_line = 1;

#ifdef FLEX_SCANNER
  if (!firstcall)
    yyrestart(yyin=f);
  else {
    firstcall = 0;
    yyin = f;
  }
#else
  yyin = f;
#endif
}

#undef yywrap
int yywrap()
{
    return 1;
}

