// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       refarray
//  TYPE:       c++ code
//  PROJECT:    lodestar
//  CONTENT:    implementation of class ReferenceArray
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================


#include "refarray.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

//----------------------------------------------------------------------------
//   class ReferenceArray
//----------------------------------------------------------------------------

//---ReferenceArray::ReferenceArray-------------------------------------------
// Input : none
// Output: none

ReferenceArray::ReferenceArray()
{
    size=0;
    allocatedSize=4;
    array = new Reference[allocatedSize];
}

//---ReferenceArray::~ReferenceArray------------------------------------------
// Input : none
// Output: none

ReferenceArray::~ReferenceArray()
{
    delete array;
}

//---Reference operator-------------------------------------------------------
// Input : int index   : get the element at this index
// Output: Reference & : the Reference-object at index

Reference &ReferenceArray::operator [](int index)
{
    assert(index>=0 && index<size);
    return array[index];
}

//---ReferenceArray::append---------------------------------------------------
// Input : Reference ref  : append the Reference-object ref to the array
// Output: none

void ReferenceArray::append(Reference ref)
{
    setSize(size+1);
    array[size-1]=ref;    
}

//---ReferenceArray::gerSize--------------------------------------------------
// Input : none
// Output: int : count of the elements in the array

int ReferenceArray::getSize()
{
    return size;
}

//---ReferenceArray::setSize--------------------------------------------------
// Input : int sizeNew : new size of the array (must be greater than the old one)
// Output: none
// resize the array - if sizeNew > allocatedSize than increase size 
// at factor two and reallocate memory

void ReferenceArray::setSize(int sizeNew)
{
    if (allocatedSize < sizeNew)
    {
        while (sizeNew > allocatedSize) allocatedSize *= 2;

        Reference *temp = new Reference[allocatedSize];
        if (temp == NULL)
        {
            fprintf(stderr,"error: insufficient memory\n");
            exit(1);
        }
    
        Reference *src = array;
        Reference *dst = temp;
        Reference *end = temp+size;
        while (dst<end) *dst++ = *src++;

        delete array;
        array = temp;
    }
    
    size = sizeNew;
}

//----------------------------------------------------------------------------
