// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       shape
//  TYPE:       c++ header
//  PROJECT:    lodestar
//  CONTENT:    class Shape, class Vertex
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================
//  MANUAL:
//
// The class Shape is an abstract superclass of Face and Line
// This is neccessary because Coords can have references to Faces and Lines
// The class Vertex is used to store the data of vertices of Faces and Lines
//
// SEE ALSO:
//     ShapeRef, Face, Line
//
// ===========================================================================

#ifndef _SHAPE_H_
#define _SHAPE_H_

#include "coord.h"
#include "material.h"
#include "normal.h"
#include "texcoord.h"

//----------------------------------------------------------------------------
//   constants for CoordSet::setShapeType 
//----------------------------------------------------------------------------

#define SHAPE_TYPE_ABSTRACT 0
#define SHAPE_TYPE_FACE     1
#define SHAPE_TYPE_LINE     2

//---forward definitions------------------------------------------------------

class Coord;   //defined in coord.h

//----------------------------------------------------------------------------
//is only a container for pointer to Coord,Material,Normal and TextureCoord
//classes used for vertices of Faces and Lines

class Vertex
{
public:

    Coord *coord;
    Material *material;
    Normal *normal;
    TextureCoord *textureCoord; 

    Vertex();
    Vertex(Coord *coo, Material *mat, Normal *nor, TextureCoord *tex);
};

//----------------------------------------------------------------------------
//an abstract superclass of Face and Line
//the done-flag is used by the method OctreeCoordSet::calcError to prevent
//accessing some shapes multiple

class Shape
{
public:

    int done;
    Shape *next;

    Shape();
};

//----------------------------------------------------------------------------

#endif
