// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       texcoord
//  TYPE:       c++ code
//  PROJECT:    lodestar
//  CONTENT:    class TextureCoordSet, class TextureCoord
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================

#include "texcoord.h"
#include "enum.h"

//----------------------------------------------------------------------------
//   class TextureCoord : Data
//----------------------------------------------------------------------------

//---TextureCoord::TextureCoord-----------------------------------------------
// Input :
// Output:

TextureCoord::TextureCoord(float ax, float ay) : Data()
{
    x=ax;
    y=ay;

    left=NULL;
    right=NULL;
}

//---TextureCoord::print------------------------------------------------------
// Input : FILE *fd             : VRML output file descriptor
//         int fid              : field id
// Output:
//write x z z separated by blanks to fd
//increase global variable global_cpl at 2. This enables the method
//DataSet::printSet making carriga returns if global_cpl >= global_floatcpl

void TextureCoord::print(FILE *fd, int fid)
{
    printFloat(fd,x); fprintf(fd," ");
    printFloat(fd,y);
    global_cpl+=2;
}

//---TextureCoord::compareWith------------------------------------------------
// Input : TextureCoord *t      :
// Output: int                  :
//compare the TextureCoord with TextureCoord t
//returns -1,0 or 1
//used for inserting in a binary tree

int TextureCoord::compareWith(TextureCoord *t)
{
    if (x<t->x) return -1;
    if (x>t->x) return 1;
    if (y<t->y) return -1;
    if (y>t->y) return 1;
    return 0;
}

//----------------------------------------------------------------------------
//   class TextureCoordSet : DataSet
//----------------------------------------------------------------------------

//---TextureCoordSet::TextureCoordSet-----------------------------------------
// Input :
// Output:

TextureCoordSet::TextureCoordSet() : DataSet()
{
    textureCoordRoot=NULL;
}

//---TextureCoordSet::~TextureCoordSet----------------------------------------
// Input :
// Output:

TextureCoordSet::~TextureCoordSet()
{
    deleteTree(textureCoordRoot);
}

//---TextureCoordSet::insert--------------------------------------------------
// Input : TextureCoord **t    :
// Output:
//first t is inserted in a binary tree using insertTree
//if an equal TextuerCoord already exists t is deleted and the pointer to t
//is redirected to the existing TextureCoord
//then a reference to t is appended to DataSet::refArray using insertArray

void TextureCoordSet::insert(TextureCoord **t)
{
    insertTree(t,&textureCoordRoot);
    insertArray(*t);
}

//---TextureCoordSet::print---------------------------------------------------
// Input : FILE *fd             : VRML output file descriptor
//         int indent           :
// Output:
//writes "point [ ... ]" to fd where ... are the TextureCoords separated
//by commas - these texture-coordinates are printed using DataSet::printSet
//additionally DataSet::setIndices is called

void TextureCoordSet::print(FILE *fd, int indent)
{
    setIndices();

    if (textureCoordRoot==NULL) return;

    printIndent(fd,indent);
    fprintf(fd,"%s [ ",fieldIdString[FID_POINT]);
    printSet(fd,indent+8,0);
    fprintf(fd," ]\n");
}

//---TextureCoordSet::insertTree----------------------------------------------
// Input : TextureCoord **t      :
//         TextureCoord **root   :
// Output:
//recursive method for inserting TextureCoords in the binary tree
//the method uses TextureCoord::compareWith for teh decision at which
//subtree t has to be inserted
//if a TextureCoord with the same specifiecations already exists the t will be
//deleted and the pointer to n is redirected to the existing TextureCord

void TextureCoordSet::insertTree(TextureCoord **t, TextureCoord **root)
{
    if ((*root)==NULL) 
    {
        //found a place to insert

        *root = *t;
        count++;
        return;
    }

    int result = (*t)->compareWith(*root);
    if (result<0) insertTree(t,&((*root)->left));
    else if (result>0) insertTree(t,&((*root)->right));

    else
    {
        //textureCoord with same specifications already exists

        delete *t;
        *t = *root;
    }
}

//---TextureCoordSet::deleteTree----------------------------------------------
// Input : TextureCoord *root    :
// Output:

void TextureCoordSet::deleteTree(TextureCoord *root)
{
    if (root == NULL) return;
    deleteTree(root->left);
    deleteTree(root->right);
    delete root;
}

//----------------------------------------------------------------------------

