// ===========================================================================
//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       texcoord
//  TYPE:       c++ header
//  PROJECT:    lodestar
//  CONTENT:    class TextureCoordSet, class TextureCoord
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================
//  MANUAL:
//
//  The class TextureCoordSet is used to store texture-coordinates defined
//  in VRML-Nodes from type TextureCoordinate2. It removes double defined
//  TextureCoords (see also DataSet). TextureCoordSet is derived from
//  DataSet and TextureCoord is derived from Data.
//  texture-coordinates which will be inserted in this class must be from
//  type TextureCoord. The texture-coordinates are inserted using the method
//  TextureCoordSet::insert(TextureCoord **t)
//  The texture-coordinates are inserted internally in a binary tree.
//  Additionally References to the texture-coordinates are appended to the
//  ReferenceArray (see also DataSet) to get the texture-coordinates
//  by index in insertion order 
//
//  SEE ALSO
//      DataSet
//
// ===========================================================================

#ifndef _TEXCOORD_H_
#define _TEXCOORD_H_

#include "dataset.h"
#include "global.h"
#include "fields.h"
#include <stdio.h>

//----------------------------------------------------------------------------

class TextureCoord : public Data
{
public:

    float x;
    float y;

    TextureCoord *left;    //pointer to left successor for binary tree
    TextureCoord *right;   //pointer to right successor for binary tree

    TextureCoord(float ax, float ay);

    virtual void print(FILE *fd, int fid);  //writes x y to fd
                                            //fid is not used
    int compareWith(TextureCoord *t);       //compare this with n - returns
                                            //-1,i 0 or 1 used for inserting
                                            //in a binary tree
};

//----------------------------------------------------------------------------

class TextureCoordSet : public DataSet
{
public:

    TextureCoordSet();
    ~TextureCoordSet();

    void insert(TextureCoord **t);    //insert t in the TextureCoordSet. t is
                                      //inserted internally in a binary tree
                                      //using insertTree. If an equal Texture-
                                      //Coord already exists t is deleted and
                                      //the pointer to t is redirected to the
                                      //existing TextureCoord. Additionally
                                      //a reference to t is appended to
                                      //DataSet::refArray
                                      
    void print(FILE *fd, int indent); //writes "point [ ... ]" to fd where ...
                                      //are the texture-coordinates separated
                                      //by commas. These texture-coordinates
                                      //are printed using DataSet::printSet
                                      //additionally DataSet::setIndices is
                                      //called
     
protected:
    TextureCoord *textureCoordRoot;  //root of the binary tree

    void insertTree(TextureCoord **t,     //insert t in the binary tree
                    TextureCoord **root); //specified by root. If an equal 
                                          //TextureCoord already exists t is
                                          //deleted and the pointer to t is
                                          //redirected to the existing
                                          //TextureCoord

    void deleteTree(TextureCoord *root);  //delete the binary tree
                                          //specified by root
};

//----------------------------------------------------------------------------

#endif

