//  Copyright 1996 Technical University of Vienna. All Rights Reserved.
// ===========================================================================
//  NAME:       tri
//  TYPE:       c++ source
//  PROJECT:    lodestar
//  CONTENT:    class Triangulator
//  VERSION:    1.0.0
// ===========================================================================
//  AUTHORS:    rs      Reinhard Sainitzer
//		hb      Herbert Buchegger
// ===========================================================================
//  HISTORY:
//
// ===========================================================================

#include "tri.h"
#include <math.h>
#include <sys/time.h>
//-----------------------------------------------------------------------------
// Triangulator
//-----------------------------------------------------------------------------

// Default value = FALSE, see main.C
int Triangulator::verbose=0;

//---Triangulator::Triangulator------------------------------------------------
// Input :
// Output:

Triangulator::Triangulator()
{
    reset();
}

//---Triangulator::~Triangulator-----------------------------------------------
// Input :
// Output:

Triangulator::~Triangulator()
{
}

//---Triangulator::reset-------------------------------------------------------
// Input :
// Output:
// Befor you can triangulate a polygon you must call this function

void Triangulator::reset()
{
    if (verbose) printf("Triangulator::reset();\n");
    count=0;
    curr=1;
    memset((void *)seg,0,sizeof(seg));

    xok=-1;
    yok=-1;
    zok=-1;

    error=0;
}

//---Triangulator::getPoint----------------------------------------------------
// Input : int row        : triangle index
//         int col        : vertices index
// Output: double & x     : x coordinate
//         double & y     : y coordinate
//         double & z     : z coordinate

void Triangulator::getPoint(int row,int col,double & x,double & y,double & z)
{
    int pos=getPoint(row,col);
    x=vec3D[pos].x;
    y=vec3D[pos].y;
    z=vec3D[pos].z;
}

//---Triangulator::addPoint----------------------------------------------------
// Input : double x,y,z   : x,y,z coordinates from the new point
// Output:

void Triangulator::addPoint(double x,double y,double z)
{
    vec3D[count].x=x;
    vec3D[count].y=y;
    vec3D[count].z=z;
    count++;
}

//---Triangulator::print2DList-------------------------------------------------
// Input : 
// Output:
// You can use the file tri_2D to draw the polygon with gnuplot
// call gnuplot and type: plot "tri_2D" with lines

void Triangulator::print2DList()
{
    int i;
    FILE * fd = fopen("tri_2D","w");
    for (i=0;i<count;i++)
    {
        if (xok) fprintf(fd," %f",vec3D[i].x);
        if (yok) fprintf(fd," %f",vec3D[i].y);
        if (zok) fprintf(fd," %f",vec3D[i].z);
        fprintf(fd,"\n");
    }
    fclose(fd);

}

//---Triangulator::print3DList-------------------------------------------------
// Input : 
// Output:
// You can use the file tri_3D to draw the polygon with gnuplot
// call gnuplot and type: splot "tri_3D" with lines

void Triangulator::print3DList()
{
    int i;
    for (i=0;i<count;i++) 
        printf("%f %f %f\n",vec3D[i].x,vec3D[i].y,vec3D[i].z);
    FILE * fd = fopen("tri_3D","w");
    for (i=0;i<count;i++) 
        fprintf(fd,"%f %f %f\n",vec3D[i].x,vec3D[i].y,vec3D[i].z);
    fclose(fd);

}

//---Triangulator::printResult-------------------------------------------------
// Input :
// Output:

void Triangulator::printResult()
{
    FILE * fd=fopen("tri_points_result","w");
    if (fd==NULL) fprintf(stderr,"error: can't create file tri_points_result for triangulation output\n");
    for (int i=0;i<getCount();i++)
    { 
        printf("result index: %d %d %d\n",triangles[i][0],triangles[i][1],triangles[i][2]);
        fprintf(fd,"%f %f\n",seg[triangles[i][0]].v0.x,seg[triangles[i][0]].v0.y);
        fprintf(fd,"%f %f\n",seg[triangles[i][1]].v0.x,seg[triangles[i][1]].v0.y);
        fprintf(fd,"%f %f\n\n",seg[triangles[i][2]].v0.x,seg[triangles[i][2]].v0.y);
    }
    fclose(fd);
}

//---Triangulator::addPoint----------------------------------------------------
// Input : double x      : x part of a 2d coordinate
//         double y      : y part of a 2d coordinate
// Output: 

void Triangulator::addPoint(double x,double y)
{
    seg[curr].is_inserted=0;   // FALSE
    seg[curr].v0.x=x;
    seg[curr].v0.y=y;
    curr++;
}

//---Triangulator::getPoint----------------------------------------------------
// Input : int row        : triangle index
//         int col        : vertices index
// Output: int            : index for the 3d array (same order as addPoint)

int Triangulator::getPoint(int row,int col)
{
    if (verbose) printf("Triangulation: getPoint %d,%d",row,col);
    if (clockwise) 
    { 
        row=count-2-row-1; 
        if (verbose) printf(" convert row (clockwise) => %d",row);
    }
    if (verbose) printf(" result=%d\n",triangles[row][col]);

    int result=triangles[row][col];
    if (count>4) result--;
    return(result);
}

//---Triangulator::getCount----------------------------------------------------
// Input :
// Output:
// result count of triangles

int Triangulator::getCount()
{
    if (count<3) return(1);
    return(count-2);    // count of triangles
}

//---Trinagulator::triangulate-------------------------------------------------
// Input :
// Output:
// Trinagulate the polygon seg[] with more then 4 vertices.
// Do not forget the clock wise order

int Triangulator::triangulate()
{
    int i;
    // WARNING: the algorithmn decrements the variable global.nseg, so it is 
    // not a good idea to use global.nseg after triangulate_monotone_polygons();
    global.nseg=count;        

    for (i=1;i<count;i++)           // necessary for Seidel's algorithmn
    {                               // seg specifies the line from point v0 to v1
        seg[i].v1.x=seg[i+1].v0.x;
        seg[i].v1.y=seg[i+1].v0.y;
    }
    seg[count].v1.x=seg[1].v0.x;
    seg[count].v1.y=seg[1].v0.y;

    if (verbose) printf("Triangulation: check clockwise order\n");
    if ((clockwise=clockWiseOrder())!=0)
    {                                              // reorder the vertices
        if (verbose) printf("wrong direction\n");
        seg[0].v1.x=seg[count].v1.x;               // seg[0] is puffer
        seg[0].v1.y=seg[count].v1.y;
        for (i=1;i<=count;i++)
        {
            seg[i].v0.x=seg[count-i].v1.x;
            seg[i].v0.y=seg[count-i].v1.y;
        }
        for (i=1;i<count;i++)
        {
            seg[i].v1.x=seg[i+1].v0.x;
            seg[i].v1.y=seg[i+1].v0.y;
        }
        seg[count].v1.x=seg[1].v0.x;
        seg[count].v1.y=seg[1].v0.y;
    }
    else if (verbose) printf("right direction\n");

    // call functions from Graphic Gems source code
    generate_random_ordering(count);
    if (construct_trapezoids(count,seg))
    {
        int nmonpoly = monotonate_trapezoids(count);
        triangulate_monotone_polygons(nmonpoly, triangles);
    }
    else return(0);

    if (verbose) printf("triangulation OK\n"); 
    return(1);
}

//---Triangulator::printList---------------------------------------------------
// Input :
// Output:

void Triangulator::printList()
{
    FILE * fd=fopen("tri_points","w");
    if (fd==NULL) fprintf(stderr,"error: can't create file tri_points for triangulation output\n");
    for (int i=1;i<=count;i++) 
        printf("%f %f\n",seg[i].v0.x,seg[i].v0.y);
    fclose(fd);
}

//---Triangulator::clockWiseOrder----------------------------------------------
// Input :
// Output:

int Triangulator::clockWiseOrder()
{
    int min=1,min2,min3; double mind=seg[1].v0.x;
    for (int i=2;i<=count;i++)
    {
        if (seg[i].v0.x<mind) 
        {
            min=i;
            mind=seg[i].v0.x;
        }
    }
    if (min==1) min3=count; else min3=min-1;
    if (min==count) min2=1; else min2=min+1;

    double x1=seg[min2].v0.x-seg[min].v0.x,
           y1=seg[min2].v0.y-seg[min].v0.y,
           x2=seg[min3].v0.x-seg[min].v0.x,
           y2=seg[min3].v0.y-seg[min].v0.y;
           
    y1=y1/sqrt(x1*x1+y1*y1);
    y2=y2/sqrt(x2*x2+y2*y2);

    return(y1>y2);
}

//---Triangulator::checkForXYZ------------------------------------------------
// Input :
// Output:
// check for plane, it make no sense to triangulate a line
// check for double points
// check for crossing

int Triangulator::checkForXYZ()
{
    if (count<4) return(1);

    if (xok==-1)
    {
        // init
        int xyok=1,yzok=1,xzok=1;
        xok=0;yok=0;zok=0;
    
        for (int i=0;i<count;i++)
        {
            // check for plane, it make no sense to triangulate a line
            if (vec3D[i].x!=vec3D[0].x) xok=1;
            if (vec3D[i].y!=vec3D[0].y) yok=1;
            if (vec3D[i].z!=vec3D[0].z) zok=1;

            if (verbose) printf("check point %d with",i);
            for (int j=i+1;j<count && !(i==0&&j==count-1);j++)
            {
                // check for double points
                if (vec3D[i].x==vec3D[j].x&&vec3D[i].y==vec3D[j].y) xyok=0;
                if (vec3D[i].x==vec3D[j].x&&vec3D[i].z==vec3D[j].z) xzok=0;
                if (vec3D[i].y==vec3D[j].y&&vec3D[i].z==vec3D[j].z) yzok=0;

                if (verbose) printf(" %d",j);

                // check for crossing, the Graphic Gems algo will terminate with
                // a segmentation fault or bus error when there are crossing lines
                // in the polygon.
                if (j>i+1)
                {
                    int jj=j+1; if (jj==count) jj=0;
                    if (checkCrossing(vec3D[i  ].x,vec3D[i  ].y, 
                                      vec3D[i+1].x,vec3D[i+1].y, 
                                      vec3D[j  ].x,vec3D[j  ].y, 
                                      vec3D[jj ].x,vec3D[jj ].y)==C_CROSSING)
                    {
                        xyok=0;
                        if (verbose) printf("crossing: xy\n");
                    }
                    if (checkCrossing(vec3D[i  ].x,vec3D[i  ].z, 
                                      vec3D[i+1].x,vec3D[i+1].z, 
                                      vec3D[j  ].x,vec3D[j  ].z, 
                                      vec3D[jj ].x,vec3D[jj ].z)==C_CROSSING)
                    {
                        xzok=0;
                        if (verbose) printf("crossing: xz\n");
                    }
                    if (checkCrossing(vec3D[i  ].y,vec3D[i  ].z, 
                                      vec3D[i+1].y,vec3D[i+1].z, 
                                      vec3D[j  ].y,vec3D[j  ].z, 
                                      vec3D[jj ].y,vec3D[jj ].z)==C_CROSSING)
                    {
                        yzok=0;
                        if (verbose) printf("crossing: yz\n");
                    }
                }
            }
            if (verbose) printf("\n");
        }  

        // join xok,yok,zok with xyok,xzok,yzok and compute result xok,yok,zok
        // when xok and yok is TRUE then zok is automatically FALSE
        if (!xok) { xyok=0; xzok=0; }
        if (!yok) { xyok=0; yzok=0; }
        if (!zok) { xzok=0; yzok=0; }
    
        xok=0; yok=0; zok=0;
        if (xyok) { xok=1; yok=1; }
        else if (xzok) { xok=1; zok=1; }
        else if (yzok) { yok=1; zok=1; }

        if (verbose) printf("tri: isok: x%d,y%d,z%d xy%d,xz%d,yz%d\n",xok,yok,zok,xyok,xzok,yzok);
        if (verbose) printf("tri: isok: x%d,y%d,z%d\n",xok,yok,zok);
    }

    if (xok+yok+zok>1) return(1);
    return(0);
}

//---Triangulator::run--------------------------------------------------------
// Input :
// Output:
// point count: 0   return NULL
//              1-2 return triangle ( 1 2 2 example)
//              3   do nothing return triangle
//              4   return two triangles ( 4 2 1 and 4 3 2 example) with a
//                  simple algorithm
//              >4  return n-2 triangles with a complex algorithm (Graphic Gems V)

int Triangulator::run()
{
    if (count==0) return(0);
    // -----------------------------------------------------------------------
    else if (count<3) 
    {
        triangles[0][0]=0;
        triangles[0][1]=count-1;
        triangles[0][2]=0;
        return(1);
    }
    // -----------------------------------------------------------------------
    else if (count==3)
    { 
        for (int i=0;i<3;i++) triangles[0][i]=i;
        return(1);
    }
    // -----------------------------------------------------------------------
    else if (count==4)
    {
        if (verbose) print3DList();
        // compute correct point pair-----------------------------------------
        if (!checkForXYZ()&&count>3)
        {
            fprintf(stderr,"error: can`t triangulate polygon (there is no correct point pair!).\n");
            return(0);
        }
        // .           .    this case would have a wrong result by simple using of 123, 234
        //  ..     . .      so determinate xmin (ymin)
        //   . .   .
        //    .  .
        //     .
        int min=0; double mind;
        if (xok) mind=vec3D[0].x; else mind=vec3D[0].y;
        for (int i=1;i<4;i++)
        {
            double value;
            if (xok) value=vec3D[i].x; else value=vec3D[i].y;
            if (value<mind) { mind=value; min=i; }
        }
        //        .  this case must be eliminated
        //    . .
        // .  .      solution: dedect case and increment min
        //    . .
        //        . 
        if (xok)
        {
            if (vec3D[(min+2)%4].x<vec3D[(min+1)%4].x || vec3D[(min+2)%4].x<vec3D[(min+3)%4].x) min=(min+1)%4;
        }
        else
            if (vec3D[(min+2)%4].y<vec3D[(min+1)%4].y || vec3D[(min+2)%4].y<vec3D[(min+3)%4].y) min=(min+1)%4;

        // compute triangle 1 and 2
        triangles[0][0]=min;
        triangles[0][1]=(min+1)%4;
        triangles[0][2]=(min+3)%4;
        triangles[1][0]=(min+1)%4;
        triangles[1][1]=(min+2)%4;
        triangles[1][2]=(min+3)%4;
        return(1);
    }
    // -----------------------------------------------------------------------
    else if (count>4)
    {
        if (verbose) print3DList();
        // compute correct point pair-----------------------------------------
        if (!checkForXYZ())
        {
            fprintf(stderr,"error: can`t triangulate polygon (error 2).\n");
            return(0);
        }
        if (verbose) print2DList();

        for (int i=0;i<count;i++)
        {
            if (xok&&yok)      addPoint(vec3D[i].x,vec3D[i].y);
            else if (xok&&zok) addPoint(vec3D[i].x,vec3D[i].z);
            else               addPoint(vec3D[i].y,vec3D[i].z);
        }
        return(triangulate());
    }
    return(1);
}

//---Triangulator::checkCrossing----------------------------------------------
// Input : double p1x - p12y   : first line
//         double p2x - p22y   : second line

int Triangulator::checkCrossing(double p1x,double p1y,double p12x,double p12y,
                                double p2x,double p2y,double p22x,double p22y)
{
    // compute 2 vectors 
    double v1x=p12x-p1x, v1y=p12y-p1y,
           v2x=p22x-p2x, v2y=p22y-p2y;

    if (v1y!=0.0) 
    { 
        double vv=v2x - v2y * v1x/v1y;
        if (vv==0.0) return(C_PARALLEL);   // parallel
        double v= ( p1x-p2x + v1x/v1y * ( p2y-p1y )) / vv;
        if (v<0.0 || v>1.0) return(C_NO_CROSSING);

        double u=(p2y + v * v2y - p1y) / v1y;
        if (u<0.0 || u>1.0) return(C_NO_CROSSING);
    }
    else if (v1x!=0.0)
    {
        double vv=v2y - v2x * v1y/v1x;
        if (vv==0.0) return(C_PARALLEL);  // parallel
        double v= ( p1y-p2y + v1y/v1x * ( p2x-p1x )) / vv;
        if (v<0.0 || v>1.0) return(C_NO_CROSSING);

        double u=(p2x + v * v2x - p1x) / v1x;
        if (u<0.0 || u>1.0) return(C_NO_CROSSING);
    }
    // else: p1 to p12 is a point 

    // printf("v = %f u=%f\n",v,u);

    return (C_CROSSING);
}
